//
// Created by TBetts on 10/14/13.
//


#import "CASwitchView.h"

@interface CASwitchView ()
{
    CAShapeLayer *_track;
    CAShapeLayer *_circle;
    UITapGestureRecognizer *_tapGestureRecognizer;
    BOOL _hasInitialized;
}
@end

@implementation CASwitchView

- (void)setFrame:(CGRect)frame
{
    [super setFrame:frame];

    if (!_hasInitialized && !CGRectEqualToRect(frame, CGRectZero))
        [self initializeControl];
    else if (_hasInitialized && !CGRectEqualToRect(frame, CGRectZero))
    {
        [self adjustControl];
    }
}

- (void)setOn:(BOOL)on
{
    _on = on;

    CGRect trackRect = CGPathGetBoundingBox(_track.path);
    CGRect circleRect = CGPathGetBoundingBox(_circle.path);

    if (_on)
    {
        _circle.position = CGPointMake(self.bounds.size.width - (circleRect.size.width + 5), CGRectGetMidY(self.bounds) - (trackRect.size.height / 2));
        _track.fillColor = [UIColor greenColor].CGColor;
    }
    else
    {
        _circle.position = CGPointMake(5, CGRectGetMidY(self.bounds) - (trackRect.size.height / 2));
        _track.fillColor = [UIColor colorWithWhite:0.95 alpha:1.0].CGColor;
    }
}

- (id)initWithFrame:(CGRect)frame
{
    self = [super initWithFrame:frame];
    if (self)
    {
        if (!_hasInitialized && !CGRectEqualToRect(frame, CGRectZero))
            [self initializeControl];
    }

    return self;
}

-(void)initializeControl
{
    _track = [CAShapeLayer layer];
    _track.path = [self generateTrackPath];
    _track.fillColor = [UIColor colorWithWhite:0.95 alpha:1.0].CGColor;
    _track.strokeColor = [UIColor colorWithWhite:0.4 alpha:1.0].CGColor;
    _track.lineWidth = 1;
    [self.layer addSublayer:_track];

    CGRect trackRect = CGPathGetBoundingBox(_track.path);
    int radius = trackRect.size.width / 3.35;

    _circle = [CAShapeLayer layer];
    _circle.path = [self generateCirclePathWithRadius:radius trackRect:trackRect];
    _circle.position = CGPointMake(5, CGRectGetMidY(self.bounds) - (trackRect.size.height / 2));
    _circle.fillColor = [UIColor colorWithWhite:0.99 alpha:1.0].CGColor;
    _circle.strokeColor = [UIColor colorWithWhite:0.5 alpha:1.0].CGColor;
    _circle.lineWidth = 1.0;
    _circle.shadowOffset = CGSizeMake(0, 2);
    _circle.shadowOpacity = 0.5;
    [self.layer addSublayer:_circle];

    _tapGestureRecognizer = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(toggleOnOff)];
    [self addGestureRecognizer:_tapGestureRecognizer];

    self.userInteractionEnabled = YES;
    _hasInitialized = YES;
}

-(void)adjustControl
{
    _track.path = [self generateTrackPath];

    CGRect trackRect = CGPathGetBoundingBox(_track.path);
    int radius = trackRect.size.width / 3.35;

    _circle.path = [self generateCirclePathWithRadius:radius trackRect:trackRect];
    if (_circle.position.x == 5)
    {
        _circle.position = CGPointMake(5, CGRectGetMidY(self.bounds) - (trackRect.size.height / 2));
    }
    else
    {
        CGRect circleRect = CGPathGetBoundingBox(_circle.path);
        _circle.position = CGPointMake(self.bounds.size.width - (circleRect.size.width + 5), CGRectGetMidY(self.bounds) - (trackRect.size.height / 2));
    }
}

-(CGPathRef)generateCirclePathWithRadius:(CGFloat)radius trackRect:(CGRect)trackRect
{
    return [UIBezierPath bezierPathWithRoundedRect:CGRectMake(0, 0, 2.0 * radius, trackRect.size.height) cornerRadius:radius].CGPath;
}

-(CGPathRef)generateTrackPath
{
    CGRect pathRect = CGRectInset(self.bounds, 10, 20);
    UIBezierPath *bezierPath = [UIBezierPath bezierPathWithRoundedRect:pathRect cornerRadius:self.bounds.size.width / 4];
    return bezierPath.CGPath;
}

-(void)toggleOnOff
{
    self.on = !self.on;
}

@end